﻿--[[---------------------------------------------------------------------------

Copyright (c) 2008 by K. Scott Piel 
All Rights Reserved

E-mail: < kscottpiel@gmail.com >
Web:    < http://www.scottpiel.com >

This file is part of nUI.

    nUI is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    nUI is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with nUI.  If not, see <http://www.gnu.org/licenses/>.
	
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

--]]---------------------------------------------------------------------------
	
-------------------------------------------------------------------------------

function nUI:battlefieldMapFix()

	if not BattlefieldMinimap or not BattlefieldMinimap.nUI_init then
		
		local wmfInit = function()
			this = WorldMapFrame;
			WorldMapFrame_OnLoad();
		end;
		
		wmfInit();
		
		if not IsAddOnLoaded( "Blizzard_BattlefieldMinimap" ) then 
		
			LoadAddOn( "Blizzard_BattlefieldMinimap" );
			
		end
	
		if IsAddOnLoaded( "Blizzard_BattlefieldMinimap" ) then 
	
			BattlefieldMinimap.nUI_init = true;
			
			hooksecurefunc( "BattlefieldMinimap_Toggle",   function() nUI:onBMMClick(); end );
	
			BattlefieldMinimap:Show();
			BattlefieldMinimap:SetParent( WorldFrame );
			BattlefieldMinimap:SetFrameStrata( "BACKGROUND" );
			BattlefieldMinimap:SetFrameLevel( 4 );
			BattlefieldMinimap:EnableMouse( true );
			BattlefieldMinimapCloseButton:Hide();
			BattlefieldMinimapCorner:Hide();
			BattlefieldMinimapBackground:Hide();
			BattlefieldMinimap_SetOpacity( 0 );

			nUI:lockFrame( BattlefieldMinimap, true, nil );			
			BattlefieldMinimap:fCache_SetPoint( "TOPLEFT", nUI.info_frame, "TOPLEFT", 1, -1 );			
			
			SHOW_BATTLEFIELD_MINIMAP = 1;
	
			-- this little hack effectively prevents the user from seeing or
			-- accessing that annoying battlefield minimap tab
			
			BattlefieldMinimap.oldOnUpdate = BattlefieldMinimap:GetScript( "OnUpdate" );	
	
			BattlefieldMinimap:SetScript( "OnUpdate", 
				function()	
					
					if BattlefieldMinimap.oldOnUpdate then
						BattlefieldMinimap:oldOnUpdate();
					end
					
					if BattlefieldMinimapTab:IsShown() then
						BattlefieldMinimapTab:Hide();
					end
					
					if nUI.bmm_frame.mapOpen then
						RequestBattlefieldPositions();
						WorldMapFrame_Update();
						WorldMapButton_OnUpdate(arg1);
					end
						
				end		
			);
	
			-- size the map to fit the info frame
			
			BattlefieldMinimap:SetScale( (nUI.info_frame:GetHeight() - 8) / BattlefieldMinimap:GetHeight() * 1.05 );
	
			-- disable the hideaway function so we can manage it in the info panel

			BattlefieldMinimap.nUI_CachedShow = BattlefieldMinimap.Show;
			BattlefieldMinimap.nUI_CachedHide = BattlefieldMinimap.Hide;
			
			BattlefieldMinimap.Show = function() end;
			BattlefieldMinimap.Hide = function() end;
			
			-- add the ability to click the battlefield minimap
			
			nUI.bmm_frame = CreateFrame( "Button", "nUI_BMM_Button", BattlefieldMinimap );
			
			nUI.bmm_frame:SetAllPoints( BattlefieldMinimap );
			nUI.bmm_frame:SetScript( "OnClick", function() self:onBMMClick(); end );
			
			nUI.bmm_frame.mapOpen = false;
			
			-- hook the world map's OnShow so if the player tries to open the map while
			-- we've stolen the detail frame, we can put it back
			
			if not nUI.bmm_frame.wmfOnShow then
				nUI.bmm_frame.wmfOnShow = WorldMapFrame:GetScript( "OnShow" );
			end
			
			WorldMapFrame:SetScript( "OnShow",
				function()
					if nUI.bmm_frame.mapOpen then nUI:onBMMClick(); end
					if nUI.bmm_frame.wmfOnShow then nUI.bmm_frame:wmfOnShow(); end
				end
			);
						
		end
	end
end

-------------------------------------------------------------------------------

function nUI:onBMMClick()

	if not nUI.bmm_frame.mapOpen and not InCombatLockdown() then

		-- we do this so that mods which hook the World Map OnShow() or
		-- the WorldMap onUpdate functions will do their thing and update
		-- the map accordingly.

		nUI.hud:Hide();
		
		WorldMapFrame:Show();
		WorldMapFrame:Hide();
		
		nUI.bmm_frame.wmdScale  = WorldMapDetailFrame:GetScale();
		nUI.bmm_frame.wmdParent = WorldMapDetailFrame:GetParent();
		nUI.bmm_frame.wmdStrata = WorldMapDetailFrame:GetFrameStrata();
		nUI.bmm_frame.wmbParent = WorldMapButton:GetParent();
		nUI.bmm_frame.mapOpen   = true;

		SetMapToCurrentZone();
		PlaySound("igQuestLogOpen");
		CloseDropDownMenus();
		
		WorldMapButton:SetParent( WorldMapDetailFrame );
		WorldMapButton:Show();
		WorldMapDetailFrame:ClearAllPoints();
		WorldMapDetailFrame:SetParent( WorldFrame );
		WorldMapDetailFrame:SetPoint( "TOP", nUI_Console, "BOTTOM", 0, -20 );
		WorldMapDetailFrame:SetPoint( "BOTTOM", nUI_Dashboard, "TOP", 0, 20 );
		WorldMapDetailFrame:SetScale( (WorldMapDetailFrame:GetTop() - WorldMapDetailFrame:GetBottom()) / WorldMapDetailFrame:GetHeight() );
		WorldMapDetailFrame:Show();
		
		UpdateMicroButtons();
		SetMapToCurrentZone();
		PlaySound("igQuestLogOpen");
		CloseDropDownMenus();
		RequestBattlefieldPositions();
		WorldMapFrame_PingPlayerPosition();

		WorldMapDetailFrame:SetScript( "OnUpdate",
			function()
				this = WorldMapFrame;
				WorldMapFrame_Update();
				RequestBattlefieldPositions();
				
				if WorldMapPlayerArrow and WorldMapPlayerArrow:GetParent() ~= WorldMapDetailFrame then
					WorldMapPlayerArrow:SetParent( WorldMapDetailFrame );
				end
				
				if PlayerArrowEffectFrame and PlayerArrowEffectFrame:GetParent() ~= WorldMapDetailFrame then
					PlayerArrowEffectFrame:SetParent( WorldMapDetailFrame );
				end
			end
		);

		WorldMapDetailFrame:SetScript( "OnKeyDown",
			function()
				local binding = GetBindingFromClick(arg1)
				if ((binding == "TOGGLEWORLDMAP") or (arg1 == "ESCAPE")) then
					RunBinding("TOGGLEWORLDMAP");
				elseif ( binding == "SCREENSHOT" ) then
					RunBinding("SCREENSHOT");
				elseif ( binding == "MOVIE_RECORDING_STARTSTOP" ) then
					RunBinding("MOVIE_RECORDING_STARTSTOP");
				end
			end		
		);

	elseif nUI.bmm_frame.mapOpen then

		nUI.hud:Show();
		
		nUI.bmm_frame.mapOpen = false;

		WorldMapDetailFrame:SetScript( "OnUpdate", nil );
		WorldMapDetailFrame:SetScript( "OnKeyDown", nil );
		
		WorldMapDetailFrame:SetParent( nUI.bmm_frame.wmdParent );
		WorldMapDetailFrame:SetScale( nUI.bmm_frame.wmdScale );
		WorldMapDetailFrame:SetFrameStrata( nUI.bmm_frame.wmdStrata );
		WorldMapDetailFrame:ClearAllPoints();
		WorldMapDetailFrame:SetPoint( "TOPLEFT", "WorldMapPositioningGuide", "TOP", -502, -69 );
		WorldMapDetailFrame:SetAlpha( 1 );
		WorldMapDetailFrame:Show();
		WorldMapButton:SetParent( nUI.bmm_frame.wmbParent );
		WorldMapButton:Show();

	end	

end

-------------------------------------------------------------------------------

function nUI:bmmHide()
	
	if not BattlefieldMinimap.nUI_CachedHide then
		if BattlefieldMinimap:IsShown() and not InCombatLockdown() then
			BattlefieldMinimap:Hide();
		end
	else
		if BattlefieldMinimap:IsShown() and not InCombatLockdown() then
			BattlefieldMinimap:nUI_CachedHide();
		end
	end

	nUI.info_frame.bmm_visible = false;
	
end

-------------------------------------------------------------------------------

function nUI:bmmShow()
	
	if not BattlefieldMinimap.nUI_CachedShow then
		if not BattlefieldMinimap:IsShown() then
			BattlefieldMinimap:Show();
		end
	else
		if not BattlefieldMinimap:IsShown() then
			BattlefieldMinimap:nUI_CachedShow();
		end
	end
	
	nUI.info_frame.bmm_visible = true;
	
end