﻿--[[---------------------------------------------------------------------------

Copyright (c) 2008 by K. Scott Piel 
All Rights Reserved

E-mail: < kscottpiel@gmail.com >
Web:    < http://www.scottpiel.com >

This file is part of nUI.

    nUI is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    nUI is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with nUI.  If not, see <http://www.gnu.org/licenses/>.
	
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

--]]---------------------------------------------------------------------------

-- basic layout

nUI_PetBarLayout =
{
	btn_size = 39,
	gap      = 1,
	anchor   = "BOTTOM",
	xOfs     = 0,
	yOfs     = 187,
};

-- anchor frame for the pet bar, stance/shapeshift bar and possess bar

nUI_PetBarAnchor = CreateFrame( "Frame", "nUI_PetBarAnchor", nUI_Dashboard, "SecureStateHeaderTemplate" );

nUI_PetBarAnchor:SetAttribute( "stateattributes",       "1:pet;2:possess;*:shapeshift" );
nUI_PetBarAnchor:SetAttribute( "attributes-pet",        "state=1" );
nUI_PetBarAnchor:SetAttribute( "attributes-shapeshift", "state=2" );
nUI_PetBarAnchor:SetAttribute( "attributes-possess",    "state=3" );
nUI_PetBarAnchor:SetAttribute( "statemap-page",         "*:$input" );

RegisterStateDriver( nUI_PetBarAnchor, "page", "[bonusbar:5] 3; [pet] 1; 2" );

-- the pet bar frame

nUI_PetBar = CreateFrame( "Frame", "nUI_PetBar", nUI_Dashboard, "SecureStateHeaderTemplate" );

nUI_PetBar:SetBackdrop(
	{
		bgFile   = "Interface\\AddOns\\nUI\\Art\\nUI_BevelboxBg.blp", 
		edgeFile = "Interface\\AddOns\\nUI\\Art\\nUI_BevelboxBorder.blp", 
		tile     = true, 
		tileSize = 1, 
		edgeSize = 12, 
		insets   = {left = 0, right = 0, top = 0, bottom = 0},
	}
);

nUI_PetBar:SetBackdropColor( 0, 0, 0, 0.35 );
nUI_PetBar:SetAttribute( "showstates", 1 );
nUI_PetBarAnchor:SetAttribute( "addchild", nUI_PetBar );

-- button initialization

nUI_PetBar.Buttons = {};

for i=1,10 do

	local button    = CreateFrame( "CheckButton", "$parent_Button"..i, nUI_PetBar, "PetActionButtonTemplate" );	

	nUI_PetBar.Buttons[i] = button;
	
	button:SetID( i );
	button:SetParent( nUI_PetBar );
	button:SetNormalTexture( "" );
	
	button.SetNormalTexture = function() end;
	
	button.actionType               = "BONUSACTIONBUTTON";
	button.actionID                 = i;
	button.nUI_CachedClearAllPoints = button.ClearAllPoints;
	button.nUI_CachedSetAllPoints   = button.SetAllPoints;
	button.nUI_CachedSetPoint       = button.SetPoint;

	button.ClearAllPoints = function() end;
	button.SetAllPoints   = function() end;
	button.SetPoint       = function() end;

	nUI_PetBar:SetAttribute( "addchild", button );
	
end

-- manage the layout of the Pet Bar

nUI_PetBar:SetScript( "OnEvent", 
	function() 

		local update_state = false;
		
		if event == "UNIT_PET" 
		and UnitIsUnit( arg1, "player" ) 
		then
			
			update_state = true;
			
		elseif event == "UNIT_FLAGS"
		or     event == "UNIT_AURA"
		then
			
			if UnitIsUnit( arg1, "pet" )
			then update_state = true;
			end
			
		elseif event == "PET_BAR_UPDATE"
		or     event == "PLAYER_CONTROL_LOST"
		or     event == "PLAYER_CONTROL_GAINED"
		or     event == "PLAYER_FARSIGHT_FOCUS_CHANGED"
		then
			
			update_state = true;

		elseif event == "VARIABLES_LOADED"
		or     event == "DISPLAY_SIZE_CHANGED"
		then
			
			nUI_PetBar:layoutFrame(); 
			
		end
		
		if update_state then
			for i in pairs( nUI_PetBar.Buttons ) do
				nUI_PetBar.updateState( i );
			end
		end		
	end 
);

nUI_PetBar:RegisterEvent( "VARIABLES_LOADED" );
nUI_PetBar:RegisterEvent( "DISPLAY_SIZE_CHANGED" );
nUI_PetBar:RegisterEvent( "PLAYER_CONTROL_LOST" );
nUI_PetBar:RegisterEvent( "PLAYER_CONTROL_GAINED" );
nUI_PetBar:RegisterEvent( "PLAYER_FARSIGHT_FOCUS_CHANGED" );
nUI_PetBar:RegisterEvent( "PET_BAR_UPDATE" );
nUI_PetBar:RegisterEvent( "UNIT_PET" );
nUI_PetBar:RegisterEvent( "UNIT_FLAGS" );
nUI_PetBar:RegisterEvent( "UNIT_AURA" );

nUI_PetBar.updateState = function( index )
	
	local button = nUI_PetBar.Buttons[index];
	local petActionsUsable = GetPetActionsUsable();
	local name, subtext, texture, isToken, isActive, 
	      autoCastAllowed, autoCastEnabled = GetPetActionInfo( index );
	
	if not isToken then
		button.layers.icon:SetTexture( texture );
		button.tooltipName = name;
	else
		button.layers.icon:SetTexture( getglobal( texture ) );
		button.tooltipName = getglobal( name );
	end
	
	button.isToken        = isToken;
	button.tooltipSubtext = subtext;
	
	if isActive then button:SetChecked( 1 );
	else button:SetChecked( 0 );
	end
	
	if name then 
		
		button:SetAlpha( 1 );
		
		if autoCastAllowed then button.layers.castable:SetAlpha( 1 ); 
		else button.layers.castable:SetAlpha( 0 ); 
		end
		
		if autoCastEnabled then button.layers.autocast:SetAlpha( 0.05 ); 
		else button.layers.autocast:SetAlpha( 0 ); 
		end
		
		if texture then
			
			if petActionsUsable then
				SetDesaturation( button.layers.icon, nil );
			else
				SetDesaturation( button.layers.icon, 1 );
			end
			
			button.layers.icon:SetAlpha( 1 );
			
		else
			
			button.layers.icon:SetAlpha( 0 );
			
		end
				
	else 
		
		button:SetAlpha( 0 );
		button.layers.castable:SetAlpha( 0 );
		button.layers.autocast:SetAlpha( 0 );
		button.layers.icon:SetAlpha( 0 );
		
	end	
end

nUI_PetBar.layoutFrame = function()
				
	nUI:setScale();

	if nUI_PetBar.fCache then nUI:lockFrame( nUI_PetBar, false ); end
				
	local btn_size    = nUI_PetBarLayout.btn_size * nUI.scale;
	local btn_gap     = nUI_PetBarLayout.gap * nUI.scale;
	local width       = 0;
	local last_button;
	
	nUI_PetBar:SetScale( 1.0 / nUI.scale );	
	nUI_PetBar:SetFrameStrata( nUI_Dashboard:GetFrameStrata() );
	nUI_PetBar:SetFrameLevel( nUI_Dashboard:GetFrameLevel()+3 );
	
	for i in pairs( nUI_PetBar.Buttons ) do
		
		local button = nUI_PetBar.Buttons[i];

		button:SetFrameStrata( nUI_PetBar:GetFrameStrata() );
		button:SetFrameLevel( nUI_PetBar:GetFrameLevel()+1 );
		button:SetScale( btn_size / button:GetHeight() );

		width = width + button:GetWidth() * button:GetEffectiveScale() + (i > 1 and btn_gap or 0);

		button:nUI_CachedClearAllPoints();
		
		if i == 1 then button:nUI_CachedSetPoint( "TOPLEFT", nUI_PetBar, "TOPLEFT", btn_size * 0.05, -btn_size * 0.05 );
		else button:nUI_CachedSetPoint( "TOPLEFT", last_button, "TOPRIGHT", btn_gap, 0 );
		end

		nUI:initActionButton( button );
		
		button.layers.autocast:Show();
		button.layers.castable:Show();
		
		nUI_PetBar.updateState( i );
		
		last_button = button;
	end		
	
	nUI_PetBar:SetHeight( btn_size * 1.1 );
	nUI_PetBar:SetWidth( width + btn_size * 0.1 );
	nUI_PetBar:SetPoint( nUI_PetBarLayout.anchor, nUI_Dashboard, "CENTER", nUI_PetBarLayout.xOfs * nUI.scale, nUI_PetBarLayout.yOfs * nUI.scale );

	nUI:lockFrame( nUI_PetBar, true, "Pet/Stance/Shapeshift Bar" );
end