﻿--[[---------------------------------------------------------------------------

Copyright (c) 2008 by K. Scott Piel 
All Rights Reserved

E-mail: < kscottpiel@gmail.com >
Web:    < http://www.scottpiel.com >

This file is part of nUI.

    nUI is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    nUI is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with nUI.  If not, see <http://www.gnu.org/licenses/>.
	
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

--]]---------------------------------------------------------------------------

local nUI_RaidRoster = {};

-- raid panel

function nUI:createRaidInfo()
	
	if not nUI.raid_frame then
		
		nUI.raid_frame  = CreateFrame( "Frame", "nUI_RaidFrame", nUI.info_frame, "SecureFrameTemplate" );
		nUI.raid_anchor = CreateFrame( "Frame", "nUI_RaidFrameAnchor", nUI_Dashboard );
		
		nUI.raid_anchor:SetScript( "OnEvent", function() nUI:raidRosterUpdate(); end );
		nUI.raid_anchor:RegisterEvent( "RAID_ROSTER_UPDATE" );
		
		nUI.info_frame:SetAttribute( "addchild", nUI.raid_frame );
		nUI.raid_frame:SetAttribute( "showstates", 3 );
		
		nUI:addInfoMode( "Raid", 3 );
		nUI:raidRosterUpdate();	
	end
	
	nUI.raid_frame:SetWidth( (nUI.info_frame:GetWidth()-3) * 3 );
	nUI.raid_frame:SetHeight( (nUI.info_frame:GetHeight()-3) * 3 );
	nUI.raid_frame:SetScale( 1/3 );
	nUI.raid_frame:SetPoint( "CENTER", nUI.info_frame, "CENTER", 0, 0 );
	nUI.raid_frame.group = {};

	nUI:addInfoMode( "Raid", 3 );
	
	local unit_height = nUI.raid_frame:GetHeight() / 8 + 4;
	local unit_width  = (nUI.raid_frame:GetWidth() - 20) / 5;
	local raid_layout =
	{
		name               = "nUI_RaidInfo",
		class_colors       = "yes",
		orientation        = "left",
		show_detail        = "no",
		show_border        = "no",
		
		Portrait =
		{
			visible   = "no",
		},
		StatusBars =
		{
			visible = "yes",
			spell   = "none",
			percent = "none",
			values  = "none",
			total   = "none",
		},
		Size =
		{
			width         = unit_width,
			height        = unit_height,
			compact       = "yes",
		},
		Origin =
		{
			anchor      = "BOTTOMLEFT",
		},
	};
	
	for i=1,8 do
		
		local raid_group = nUI.raid_frame.group[i];
		
		if not raid_group then 
			raid_group              = CreateFrame( "Frame", "$parent_Group"..i, nUI.raid_frame );	
			raid_group.label        = raid_group:CreateFontString( "$parentLabel" );
			nUI.raid_frame.group[i] = raid_group;
			raid_group.unit         = {};
		end
		
		if not InCombatLockdown() then raid_group:Hide(); end
		
		raid_group.group_id = i;
		raid_group:SetWidth( nUI.raid_frame:GetWidth() );
		raid_group:SetHeight( unit_height - 4 );

		raid_group.label:SetPoint( "LEFT", raid_group, "LEFT", 0, 0 );
		raid_group.label:SetFont( "Fonts\\ARIALN.TTF", unit_height * 0.5, "OUTLINE" );
		raid_group.label:SetJustifyH( "LEFT" );
		raid_group.label:SetJustifyV( "MIDDLE" );
		raid_group.label:SetTextColor( 1, 0.83, 0, 1 );
		raid_group.label:SetText( i );

		if i == 1 then 
			raid_group:SetPoint( "TOPLEFT", nUI.raid_frame, "TOPLEFT", 0, 0 );
		else
			raid_group:SetPoint( "TOP", nUI.raid_frame.group[i-1], "BOTTOM", 0, 0 );
		end
		
		local last_unit = nil;
		
		for j=1,5 do
			
			raid_layout.id   = nUI_RaidRoster[i][j];
			local id         = "raid"..raid_layout.id;
			local new_layout = nUI.Class.UnitLayout:new( id, raid_layout );
			local unit       = raid_group.unit[j];
			
			if not unit then
				unit = nUI.Class.Unit:new( raid_group, raid_layout.id, new_layout );
				raid_group.unit[j] = unit;
			else
				unit:layoutFrame( new_layout );
			end
			
			unit.frame:UnregisterEvent( "RAID_ROSTER_UPDATE" );
			unit:ClearAllPoints();
			
			if j == 1 then unit:SetPoint( "LEFT", raid_group, "LEFT", 10, 1 );
			else unit:SetPoint( "LEFT", last_unit.frame, "RIGHT", 0, 0 );
			end
		
			last_unit = unit;
		end	
		
		if not raid_group:IsShown() then raid_group:Show(); end
		
	end
	
	nUI.raid_frame.initialized = true;
	
end

-------------------------------------------------------------------------------

function nUI:raidRosterUpdate()
	
	-- create a list of valid raid IDs
	
	local nUI_RaidID = {};
	
	for i=1,40 do nUI_RaidID[i] = 1; end
	
	-- create and empty raid roster
	
	local next_id  = {};	
	nUI_RaidRoster = {};
	
	for i=1,8 do 
		next_id[i]        = 1;
		nUI_RaidRoster[i] = {}; 
	end
	
	-- get the raid roster info
	
	for i=1,GetNumRaidMembers() do
		
		local	name, rank, subGroup = GetRaidRosterInfo( i );		
		local	ndx = next_id[subGroup];
		
		next_id[subGroup]             = ndx+1;		
		nUI_RaidRoster[subGroup][ndx] = i;
		nUI_RaidID[i]                 = nil;
		
	end
	
	-- fill out the rest of the raid roster with the unused
	-- raid IDs
	
	local ndx = 1;
	
	for i=1,8 do
		for j=1,5 do
			
			if not nUI_RaidRoster[i][j] then
				
				while not nUI_RaidID[ndx] do
					ndx = ndx+1;
				end
				
				nUI_RaidRoster[i][j] = ndx;
				
				ndx = ndx+1;
				
			end
		end
	end	
	
	-- if we're already up and running, then update all of the raid unit frames
	
	if nUI.raid_frame.initialized then
		
		for i=1,8 do

			local raid_group = nUI.raid_frame.group[i];
			
			for j=1,5 do
				
				local id = nUI_RaidRoster[i][j];
			
				raid_group.unit[j]:setRaidID( "raid"..id, id );
				
			end
		end
	end
end